﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Text.RegularExpressions;
using System.Windows;
using System.Windows.Browser;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using Ozeki.MediaGateway;

namespace _04_Webphone_Silverlight_SIP_CMS
{
    public partial class MainPage : UserControl
    {
        MediaConnection connection;
        MediaStreamSender streamSender;
        MediaStreamReceiver streamReceiver;
        AudioPlayer audioPlayer;
        Microphone microphone;

        PhoneState phoneState;

        public MainPage()
        {
            InitializeComponent();
            HtmlPage.RegisterScriptableObject("MainPage", this);

            connection = new MediaConnection("localhost:4502/SIPGateway");
            connection.ConnectionStateChanged += connection_ConnectionStateChanged;
            connection.Connect();
        }

        [ScriptableMember]
        public void SetPhoneState(PhoneState newState)
        {
            switch (newState)
            {
                case PhoneState.Ready:
                    HtmlPage.Window.Invoke("SetActionButtonState", 0, true, true);
                    break;
                case PhoneState.InRinging:
                    HtmlPage.Window.Invoke("SetActionButtonState", 1, true, true);
                    break;
                case PhoneState.OutRinging:
                    HtmlPage.Window.Invoke("SetActionButtonState", 1, false, true);
                    break;
                case PhoneState.InCall:
                    HtmlPage.Window.Invoke("SetActionButtonState", 1, false, true);
                    break;
            }
            phoneState = newState;
        }

        /// <summary>
        /// Indicates the connectionstate has been changed with server.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void connection_ConnectionStateChanged(object sender, GenericEventArgs<ConnectionState> e)
        {
            switch (e.Item)
            {
                case ConnectionState.Success:
                    InvokeGUIThread(() =>
                    {
                        HtmlPage.Window.Invoke("ConnectionStateChanged", "Online");
                    });
                    phoneState = PhoneState.Ready;
                    audioPlayer = new AudioPlayer();
                    connection.Client = this;
                    break;
                default:
                    InvokeGUIThread(() =>
                    {
                        HtmlPage.Window.Invoke("ConnectionStateChanged", "Connection failed");
                        HtmlPage.Window.Invoke("SetActionButtonState", 0, false, false);
                    });
                    break;
            }
        }

        /// <summary>
        /// Start a call or accept a call request.
        /// </summary>
        [ScriptableMember]
        public void btnCall_Click(string dialNumber)
        {
            btnMicAccess.Visibility = System.Windows.Visibility.Visible;
            if (Microphone.GetPermissionToMicrophone())
            {
                ReleaseStreams();

                switch (phoneState)
                {
                    case PhoneState.InRinging:
                        connection.InvokeOnConnection("Accept");
                        SetPhoneState(PhoneState.InCall);
                        break;
                    case PhoneState.Ready:
                        if (string.IsNullOrWhiteSpace(dialNumber))
                        {
                            HtmlPage.Window.Invoke("UpdateInfo", "Please give a phone number");
                            return;
                        }
                        dialNumber = Regex.Replace(dialNumber, "\\s", "");
                        connection.InvokeOnConnection("Call", dialNumber);
                        HtmlPage.Window.Invoke("UpdateInfo", "Outgoing call progress.");
                        SetPhoneState(PhoneState.OutRinging);
                        break;
                }
            }
            else
            {
                MessageBox.Show("Please, grant permission to the microphone.");
            }
        }

        /// <summary>
        /// befejezi a folyamatban lévő hívást, vagy a bejövő híjvást elutasítja.
        /// </summary>
        [ScriptableMember]
        public void btnStopCall_Click()
        {
            switch (phoneState)
            {
                case PhoneState.InRinging:
                    connection.InvokeOnConnection("Reject");
                    SetPhoneState(PhoneState.Ready);
                    break;
                case PhoneState.OutRinging:
                case PhoneState.InCall:
                    HtmlPage.Window.Invoke("UpdateInfo", "Call stop, ready to call.");
                    connection.InvokeOnConnection("HangUp");
                    SetPhoneState(PhoneState.Ready);
                    break;
            }

            HtmlPage.Window.Invoke("ResetDialNumber");
        }


        [ScriptableMember]
        public void UpdateText(string text)
        {
            MessageBox.Show(text);
        }

        /// <summary>
        /// Sets the client status. 
        /// </summary>
        /// <param name="isReady"></param>
        /// <param name="name"></param>
        public void OnSetReadyStatus(bool isReady, string name)
        {
            HtmlPage.Window.Invoke("SetActionButtonState", 0, isReady, isReady);

            string info = isReady ? "Ready to call." : "Offline";
            HtmlPage.Window.Invoke("SetReadyStatus", info, name);

            //  txtboxInfo.Text = isReady ? "Ready to call." : "Offline";
        }


        /// <summary>
        /// Indicates the incoming call request from the given other client.
        /// </summary>
        /// <param name="remotpartyId"> The request owner's client id.</param>
        public void OnCallRequest(string remotpartyId)
        {
            SetPhoneState(PhoneState.InRinging);
            InvokeGUIThread(() =>
            {
                HtmlPage.Window.Invoke("UpdateInfo", "Call received from " + remotpartyId);
            });
        }


        /// <summary>
        /// Starts to send the audio data to the server and starts to play other party's media data.
        /// </summary>
        /// <param name="publisStreamName"></param>
        /// <param name="playStreamName"></param>
        public void OnInCall(string publisStreamName, string playStreamName)
        {
            streamSender = new MediaStreamSender(connection);
            microphone = Microphone.GetMicrophone();
            streamSender.AttachMicrophone(microphone);
            streamSender.Publish(publisStreamName);

            streamReceiver = new MediaStreamReceiver(connection);
            audioPlayer.AttachMediaStreamReceiver(streamReceiver);
            streamReceiver.Play(playStreamName);
        }

        /// <summary>
        ///  Indicates an error.
        /// </summary>
        /// <param name="callError"></param>
        public void OnCallErrorOccurred(CallError callError)
        {
            InvokeGUIThread(() =>
            {
                HtmlPage.Window.Invoke("UpdateInfo", callError.ToString());
                SetPhoneState(PhoneState.Ready);
            });
        }

        /// <summary>
        /// Indicates the call state has changed.
        /// </summary>
        /// <param name="callState">The current state of the call.</param>
        public void OnCallStateChanged(CallState callState)
        {
            InvokeGUIThread(() =>
            {
                HtmlPage.Window.Invoke("UpdateInfo", callState.ToString());
                switch (callState)
                {
                    case CallState.InCall:
                        SetPhoneState(PhoneState.InCall);
                        break;
                    case CallState.Cancelled:
                        SetPhoneState(PhoneState.Ready);
                        HtmlPage.Window.Invoke("ResetDialNumber");
                        break;
                    case CallState.Completed:
                        SetPhoneState(PhoneState.Ready);
                        HtmlPage.Window.Invoke("ResetDialNumber");
                        ReleaseStreams();
                        break;
                }
            });
        }

        /// <summary>
        /// Release MediaStreamSender or MediaStreamReceiver object if it necessary.(All started media call action required new MediaStreamSender and MediaStreamReciver object.)
        /// </summary>
        private void ReleaseStreams()
        {
            if (streamSender != null)
            {
                streamSender.Close();
            }
            if (streamReceiver != null)
            {
                streamReceiver.Close();
            }
        }


        /// <summary>
        /// The controls of the Windows Form applications can only be modified on the GUI thread. This method grants access to the GUI thread.
        /// </summary>
        /// <param name="action"></param>
        private void InvokeGUIThread(Action action)
        {
            Dispatcher.BeginInvoke(action);
        }


        /// <summary>
        /// erre azért van szükség mert a silverlight security policy kizárólag silverlight alkalmazás alól történő user interakció esetén dobja fel a mikrofon eléréshez szükséges ablakot
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnMicAccess_Click(object sender, RoutedEventArgs e)
        {
            Microphone.GetPermissionToMicrophone();
        }

    }
}
